## PROJECT OVERVIEW
* Type: Diploma thesis
* Subject: Development of the SoftLab UserBase (SLUB) web application [http://artemis.cslab.ece.ntua.gr:8080/jspui/handle/123456789/17403 and https://dspace.lib.ntua.gr/xmlui/handle/123456789/50498]
* Author: Georgios Kazelidis \<giorgkazelidis@gmail.com>
* Supervisor: Nikolaos Papaspyrou \<nickie@softlab.ntua.gr>
* Copyright: © 2019 Georgios A. Kazelidis for SoftLab
* License: GNU General Public License, version 2 or (at your option) any later version [https://www.gnu.org/licenses/old-licenses/gpl-2.0.html]

## ENVIRONMENT SPECIFICATIONS
* Working OS: Ubuntu 16.04 LTS [https://www.ubuntu.com/]
* Python version: 3.5.2 (pre-installed) [https://www.python.org/]
* Git version: 2.7.4 [https://git-scm.com/]
* pip3 version: 10.0.1 [https://pypi.org/project/pip/]
* virtualenv version: 15.2.0 [https://pypi.org/project/virtualenv/]
* Django version: 2.0.4 [https://www.djangoproject.com/]
* pytz version: 2018.4 [https://pypi.org/project/pytz/]
* concurrent-log-handler version: 0.9.12 [https://pypi.org/project/concurrent-log-handler/]
* mod_wsgi version: 4.6.5 [https://pypi.org/project/mod_wsgi/]
* mysqlclient version: 1.3.12 [https://pypi.org/project/mysqlclient/]
* MySQL (community) server version: 5.7.24 [https://dev.mysql.com/]
* Apache web server version: 2.4.18 [https://httpd.apache.org/]

## DOWNLOADING THE PROJECT & SETTING UP THE ENVIRONMENT
(A) install pip3 (`pip3 --version` to see if it is already installed):

    sudo apt install python3-pip
(B) install virtualenv (`virtualenv --version` to see if it is already installed):

    sudo apt install virtualenv
(C\) install and configure Git [sources:  
&nbsp; &nbsp; &nbsp; -- "How To Install Git with Apt" and "How To Set Up Git" sections of https://www.digitalocean.com/community/tutorials/how-to-install-git-on-ubuntu-16-04  
&nbsp; &nbsp; &nbsp; -- https://www.tutorialspoint.com/git/git_environment.htm  
&nbsp; &nbsp; &nbsp; ]:  
&nbsp; &nbsp; &nbsp; (1) install Git (`git --version` to see if it is already installed):

    sudo apt install git
&nbsp; &nbsp; &nbsp; (2) set the Git "name" and "email" global parameters (`git config --list` to see if they have already been set):

    git config --global user.name "YOUR_NAME"
    git config --global user.email "YOUR_EMAIL"
(D) create a local instance of the remote repository (userbase) [sources:  
&nbsp; &nbsp; &nbsp; -- https://git-scm.com/docs/git-clone  
&nbsp; &nbsp; &nbsp; ]:  
&nbsp; &nbsp; &nbsp; (1) navigate to the directory (LOCAL_REPO_CONTAINER_DIR) where you would like to clone the repository (depending on the directory you chose to navigate to, you may need to run shell commands with `sudo` from here onwards to avoid raising "Permission Denied" exceptions):

    cd LOCAL_REPO_CONTAINER_DIR
&nbsp; &nbsp; &nbsp; (2) clone the repository to create the corresponding instance locally:

    git clone https://git.softlab.ntua.gr/giorgkazelidis/userbase.git
&nbsp; &nbsp; &nbsp; (3) navigate to the local instance:

    cd userbase
(E) install and configure Django and MySQL inside the local instance using virtualenv with Python 3 [sources:  
&nbsp; &nbsp; &nbsp; -- steps 2 and 3 of https://help.dreamhost.com/hc/en-us/articles/215317948-How-to-install-Django-using-virtualenv  
&nbsp; &nbsp; &nbsp; -- step 3 of https://www.digitalocean.com/community/tutorials/how-to-create-a-django-app-and-connect-it-to-a-database  
&nbsp; &nbsp; &nbsp; -- end of section http://docs.python-guide.org/en/latest/dev/virtualenvs/#lower-level-virtualenv  
&nbsp; &nbsp; &nbsp; -- https://stackoverflow.com/questions/41060382/using-pip-to-install-packages-to-anaconda-environment  
&nbsp; &nbsp; &nbsp; -- https://docs.djangoproject.com/en/2.0/topics/install/  
&nbsp; &nbsp; &nbsp; -- https://pypi.org/project/mysqlclient/  
&nbsp; &nbsp; &nbsp; -- https://pypi.org/project/concurrent-log-handler/  
&nbsp; &nbsp; &nbsp; -- https://pypi.org/project/mod_wsgi/  
&nbsp; &nbsp; &nbsp; -- https://www.digitalocean.com/community/tutorials/how-to-install-mysql-on-ubuntu-16-04  
&nbsp; &nbsp; &nbsp; -- https://dev.mysql.com/doc/refman/5.7/en/mysql-tzinfo-to-sql.html  
&nbsp; &nbsp; &nbsp; -- https://datawookie.netlify.com/blog/2017/08/setting-up-time-zones-in-mysql/  
&nbsp; &nbsp; &nbsp; -- "Create a MySQL Database" section of http://www.marinamele.com/taskbuster-django-tutorial/install-and-configure-mysql-for-django  
&nbsp; &nbsp; &nbsp; -- https://gist.github.com/hofmannsven/9164408  
&nbsp; &nbsp; &nbsp; -- https://dev.mysql.com/doc/refman/5.7/en/sql-syntax.html  
&nbsp; &nbsp; &nbsp; -- https://stackoverflow.com/questions/43644218/why-is-table-charset-set-to-utf8mb4-and-collation-to-utf8mb4-unicode-520-ci?rq=1  
&nbsp; &nbsp; &nbsp; -- https://www.tutorialspoint.com/django/django_admin_interface.htm  
&nbsp; &nbsp; &nbsp; -- https://docs.djangoproject.com/en/2.0/ref/django-admin/  
&nbsp; &nbsp; &nbsp; -- https://docs.python.org/3/tutorial/interpreter.html  
&nbsp; &nbsp; &nbsp; -- https://stackoverflow.com/questions/436198/what-is-an-alternative-to-execfile-in-python-3  
&nbsp; &nbsp; &nbsp; -- https://docs.python.org/3.5/tutorial/inputoutput.html#reading-and-writing-files  
&nbsp; &nbsp; &nbsp; ]:  
&nbsp; &nbsp; &nbsp; (1) create the virtual environment (venv) and specify the desired Python version to be used:

    virtualenv venv -p $(which python3)
&nbsp; &nbsp; &nbsp; (2) activate the virtual environment:

    source venv/bin/activate
&nbsp; &nbsp; &nbsp; (3) install necessary Python and MySQL development headers and libraries (`apt list --installed | grep -E "python3-dev|libmysqlclient-dev"` to see if they are already installed):

    sudo apt install python3-dev libmysqlclient-dev
&nbsp; &nbsp; &nbsp; (4) install appropriate versions of virtual-environment-specific packages:

    $(which pip) install -r requirements.txt
&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; ALTERNATIVELY, you could install the packages (Django, mysqlclient, concurrent-log-handler, mod_wsgi and pytz - the last one is installed alongside Django) individually by running the following commands:

    $(which pip) install Django==2.0.4
    $(which pip) install mysqlclient==1.3.12
    $(which pip) install concurrent-log-handler==0.9.12
    $(which pip) install mod_wsgi==4.6.5
&nbsp; &nbsp; &nbsp; (5) install MySQL server (`mysqld --version` to see if it is already installed):  
&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; * ATTENTION: while installing MySQL server, you will be asked to set the password for the root user.

    sudo apt install mysql-server
    mysql_secure_installation
&nbsp; &nbsp; &nbsp; (6) use the password that was set for the root user during MySQL server installation as the DATABASES['default']['PASSWORD'] value in **LOCAL_REPO_CONTAINER_DIR/userbase/slub/slub/settings.py**  
&nbsp; &nbsp; &nbsp; (7) ensure that MySQL server is active by running `sudo systemctl status mysql` (if it is not, start it by running `sudo systemctl start mysql`) and `mysqladmin -u root -p version` (you will be asked to provide the password of the MySQL root user)  
&nbsp; &nbsp; &nbsp; (8) copy the time zone definitions of the system's zoneinfo database, which is located in the /usr/share/zoneinfo/ directory of Ubuntu and most other Unix-like systems, to the "mysql" database without worrying about some possible "Unable to load/Skipping" warnings (you will be asked to provide the password of the MySQL root user) and restart MySQL server (you can skip this substep if you have already executed it once):

    mysql_tzinfo_to_sql /usr/share/zoneinfo | mysql -u root -p mysql
    sudo systemctl restart mysql
&nbsp; &nbsp; &nbsp; (9) navigate to the Django project directory (slub):

    cd slub
&nbsp; &nbsp; &nbsp; (10) open MySQL console with root privileges:

    python manage.py dbshell
&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; ALTERNATIVELY, you could run `mysql -u root -p` by providing the password of the MySQL root user  
&nbsp; &nbsp; &nbsp; (11) if the project database (usermergeDB) has already been created (`SHOW DATABASES;` to check), delete it:

    DROP DATABASE usermergeDB;
&nbsp; &nbsp; &nbsp; (12) create the project database (no tables are created during this substep) with "utf8mb4" as the character set and X as the collation, where X is "utf8mb4_0900_ai_ci" for MySQL (server version) 8.0, "utf8mb4_unicode_520_ci" for MySQL 5.6/5.7 or "utf8mb4_unicode_ci" for MySQL 5.5:

    CREATE DATABASE usermergeDB CHARACTER SET utf8mb4 COLLATE X;
&nbsp; &nbsp; &nbsp; (13) exit MySQL console:

    EXIT;
&nbsp; &nbsp; &nbsp; (14) use the collation that was selected during the project database creation as the DATABASES['default']['OPTIONS']['init_command']#collation_connection value in **LOCAL_REPO_CONTAINER_DIR/userbase/slub/slub/settings.py**  
&nbsp; &nbsp; &nbsp; (15) if MySQL server version is 5.5.4 and earlier, set SET_DEFAULT_STORAGE_ENGINE_TO_INNODB to "True" in **LOCAL_REPO_CONTAINER_DIR/userbase/slub/slub/settings.py**  
&nbsp; &nbsp; &nbsp; (16) initiate the project database (the tables are created during this substep):

    python manage.py migrate
&nbsp; &nbsp; &nbsp; (17) if you have previously set SET_DEFAULT_STORAGE_ENGINE_TO_INNODB to "True", set it back to "False"  
&nbsp; &nbsp; &nbsp; (18) populate the project database appropriately, e.g. by loading a Django fixture (see Django's `dumpdata` and `loaddata` administrative commands at https://docs.djangoproject.com/en/2.0/ref/django-admin/) or MySQL dump file (see https://dev.mysql.com/doc/refman/5.7/en/backup-and-recovery.html) into the database, by executing the populateDB.py script of the application directory (usermerge) via the Python interpreter (run the following command), etc.:

    python manage.py shell --command="with open('./usermerge/populateDB.py') as popDB: exec(popDB.read())"
&nbsp; &nbsp; &nbsp; (19) deactivate the virtual environment:

    deactivate

## ACCESSING THE DJANGO ADMIN SITE OR RUNNING THE PROJECT APPLICATION (AFTER STARTING THE DJANGO DEVELOPMENT SERVER INSIDE THE VIRTUAL ENVIRONMENT)
(A) activate the virtual environment of the local repository instance:

    cd LOCAL_REPO_CONTAINER_DIR/userbase
    source venv/bin/activate
(B) navigate to the Django project directory:

    cd slub
(C\) create a superuser account in the project database if you have not already done so (this step is necessary for accessing the Django admin site) [sources:  
&nbsp; &nbsp; &nbsp; -- https://docs.djangoproject.com/en/2.0/ref/contrib/admin/  
&nbsp; &nbsp; &nbsp; ]:  
&nbsp; &nbsp; &nbsp; * ATTENTION: while creating the superuser account, you will be asked to set the corresponding credentials (username and password) and (optional) email.

    python manage.py createsuperuser
(D) start the Django development server (at http://127.0.0.1:8000/):

    python manage.py runserver
(E) access the Django admin site by navigating to http://127.0.0.1:8000/django-admin/ (via web browser) and filling out the emerging login form with the credentials of the aforementioned superuser account OR run the project application (usermerge) by navigating to http://127.0.0.1:8000/

## STOPPING THE DJANGO DEVELOPMENT SERVER (AND EXITING THE VIRTUAL ENVIRONMENT)
(A) stop the Django development server by returning to the terminal where you have run `python manage.py runserver` and pressing `Ctrl-C`  
(B) deactivate the virtual environment:

    deactivate

## INSTALLING AND CONFIGURING THE APACHE WEB SERVER TO DEPLOY THE PROJECT
[sources:  
&nbsp; &nbsp; &nbsp; -- https://www.digitalocean.com/community/tutorials/how-to-install-the-apache-web-server-on-ubuntu-16-04  
&nbsp; &nbsp; &nbsp; -- https://pypi.org/project/mod_wsgi/  
&nbsp; &nbsp; &nbsp; -- https://www.digitalocean.com/community/tutorials/how-to-set-up-a-firewall-with-ufw-on-ubuntu-16-04  
&nbsp; &nbsp; &nbsp; -- https://docs.djangoproject.com/en/2.0/howto/deployment/wsgi/modwsgi/  
&nbsp; &nbsp; &nbsp; -- https://modwsgi.readthedocs.io/en/develop/user-guides/quick-configuration-guide.html  
&nbsp; &nbsp; &nbsp; -- https://www.digitalocean.com/community/questions/apache2-www-and-subsequent-folder-permissions  
&nbsp; &nbsp; &nbsp; -- https://askubuntu.com/questions/767504/permissions-problems-with-var-www-html-and-my-own-home-directory-for-a-website  
&nbsp; &nbsp; &nbsp; -- https://webmasters.stackexchange.com/questions/83633/have-disabled-apache-site-config-file-000-default-conf-but-it-still-seems-activ  
]:  
(A) install Apache web server and development package (`apache2 -v` and `apt list --installed | grep "apache2-dev"` to see if they are already installed):

    sudo apt install apache2 apache2-dev
(B) activate and adjust UFW firewall to open only port 80 for unencrypted incoming web traffic (`sudo ufw status verbose` to check the firewall status):

    sudo ufw enable
    sudo ufw allow 'Apache'
(C\) ensure that the web server is active by running `sudo systemctl status apache2` (if it is not, start it by running `sudo systemctl start apache2`) and navigating to http://SERVER_IP via web browser (`hostname -I` to specify SERVER_IP)  
(D) navigate to the default VHost DocumentRoot of Apache (/var/www/html directory):

    cd /var/www/html
(E) create the project deployment directory (softlab_userbase) that will contain the project document (DocumentRoot) and script directories to be deployed and navigate to it:

    sudo mkdir softlab_userbase
    cd softlab_userbase
(F) create the empty document directory (public_docs):

    sudo mkdir public_docs
(G) follow all the steps of the "DOWNLOADING THE PROJECT & SETTING UP THE ENVIRONMENT" section above - at the end, a local instance of the remote repository (userbase) should have been created in the deployment directory and should serve as the script directory  
(H) in **/var/www/html/softlab_userbase/userbase/slub/slub/settings.py**, replace the current value of SECRET_KEY with a similar, confidential one and ensure that DEBUG and ALLOWED_HOSTS are "False" and "['*']" respectively (if they are not, set them appropriately)  
(I) navigate to the deployment directory:

    cd /var/www/html/softlab_userbase
(J) change the user and group owners of the script directory to "www-data" (Apache system user/group) - `ls -l` to verify the change:

    sudo chown www-data:www-data userbase
(K) navigate to the Apache ServerRoot (/etc/apache2 directory):

    cd /etc/apache2
(L) specify the Apache configuration directive needed to load the mod_wsgi module from the virtual environment (venv) of the script directory by running `mod_wsgi-express module-config` inside the aforementioned virtual environment and keeping the "LoadModule" line of the corresponding output:

    source /var/www/html/softlab_userbase/userbase/venv/bin/activate
    mod_wsgi-express module-config
    deactivate
(M) use a text editor, e.g. gedit, and the specified module loading directive to create the mod_wsgi loading file (wsgi.load) in the module loading directory (mods-available):

    sudo gedit mods-available/wsgi.load
with the following content:

	# For more information, see the /var/www/html/softlab_userbase/userbase/README.md file as well as the following links:
	# * https://httpd.apache.org/docs/current/mod/mod_so.html
	# * https://pypi.org/project/mod_wsgi/

	SET_THE_SPECIFIED_MODULE_LOADING_DIRECTIVE_HERE
(N) activate the mod_wsgi module loading and restart the web server (at http://SERVER_IP) to apply it:

    sudo a2enmod wsgi
    sudo systemctl restart apache2
(O) use a text editor, e.g. gedit, to create the project deployment configuration file (softlab_userbase.conf) in the VHost configuration directory (sites-available):

    sudo gedit sites-available/softlab_userbase.conf
with the following content (make sure to replace SERVER_IP, PROCESSES_NUMBER, THREADS_NUMBER and python3.V with the appropriate values, e.g. run `hostname -I` to specify SERVER_IP and `python3 --version` to specify python3.V - in the current context, it is python3.5):

	# For more information, see the /var/www/html/softlab_userbase/userbase/README.md and /etc/apache2/sites-available/000-default.conf files
	# as well as the following links:
	# * https://httpd.apache.org/docs/current/mod/core.html
	# * https://httpd.apache.org/docs/current/logs.html
	# * https://modwsgi.readthedocs.io/en/develop/configuration.html
	# * https://modwsgi.readthedocs.io/en/develop/user-guides/quick-configuration-guide.html
	# * https://modwsgi.readthedocs.io/en/develop/user-guides/virtual-environments.html
	# * https://docs.djangoproject.com/en/2.0/howto/deployment/wsgi/modwsgi/

	<VirtualHost *:80>
		ServerName SERVER_IP
		ServerAlias slub.softlab.ntua.gr slub.softlab.ece.ntua.gr
		ServerAdmin webmaster@courses.softlab.ntua.gr

		DocumentRoot /var/www/html/softlab_userbase/public_docs

		Alias /static/admin/ /var/www/html/softlab_userbase/userbase/venv/lib/python3.V/site-packages/django/contrib/admin/static/admin/
		Alias /static/ /var/www/html/softlab_userbase/userbase/slub/usermerge/static/

		WSGIDaemonProcess softlab_userbase processes=PROCESSES_NUMBER threads=THREADS_NUMBER display-name=%{GROUP} \
			python-home=/var/www/html/softlab_userbase/userbase/venv python-path=/var/www/html/softlab_userbase/userbase/slub
		WSGIProcessGroup softlab_userbase
		WSGIApplicationGroup %{GLOBAL}

		WSGIScriptAlias / /var/www/html/softlab_userbase/userbase/slub/slub/wsgi.py

		ErrorLog ${APACHE_LOG_DIR}/error.log
		CustomLog ${APACHE_LOG_DIR}/access.log combined
	</VirtualHost>
(P\) activate the project deployment configuration and reload the web server to apply it:

    sudo a2ensite softlab_userbase
    sudo systemctl reload apache2

## ACCESSING THE DJANGO ADMIN SITE OR RUNNING THE PROJECT APPLICATION (AFTER STARTING THE APACHE WEB SERVER)
(A) create a superuser account in the project database inside the virtual environment of the project script directory if you have not already done so (this step is necessary for accessing the Django admin site) - see the "ACCESSING THE DJANGO ADMIN SITE OR RUNNING THE PROJECT APPLICATION (AFTER STARTING THE DJANGO DEVELOPMENT SERVER INSIDE THE VIRTUAL ENVIRONMENT)" section above for more information:  
&nbsp; &nbsp; &nbsp; * ATTENTION: if running `python slub/manage.py createsuperuser` below raises a "Permission denied: '/var/www/html/softlab_userbase/userbase/production_logs'" PermissionError exception, create the production_logs directory as the www-data user first by running `sudo -u www-data -s mkdir production_logs` (see the "Logging" section of /var/www/html/softlab_userbase/userbase/slub/slub/settings.py) and then retry running the former.

    cd /var/www/html/softlab_userbase/userbase
    source venv/bin/activate
    python slub/manage.py createsuperuser
    deactivate
(B) start the Apache web server (`sudo systemctl status apache2` to see if it is already active):

    sudo systemctl start apache2
(C\) access the Django admin site by navigating to http://SERVER_IP/django-admin/ and filling out the emerging login form with the credentials of the aforementioned superuser account OR run the project application (usermerge) by navigating to http://SERVER_IP - the static files/directories of the project application and Django admin site are also accessible at http://SERVER_IP/static/ and http://SERVER_IP/static/admin/ respectively

## CHECKLIST AND TIPS FOR PROJECT DEPLOYMENT
* https://docs.djangoproject.com/en/2.0/howto/deployment/
* https://www.freecodecamp.org/news/django-in-the-wild-tips-for-deployment-survival-9b491081c2e4/
* https://www.agileleaf.com/blog/a-better-way-to-manage-settings-py-in-your-django-projects/
* https://simpleisbetterthancomplex.com/tips/2017/07/03/django-tip-20-working-with-multiple-settings-modules.html

## LIST OF SUGGESTED TO-DO ACTIONS
* Define a cron job that uses MySQL's `mysqldump` administrative command to regularly create project database backups (dump files) needed for recovery in case of system crashes, hardware failures, accidental data deletion, etc. [https://www.ostechnix.com/a-beginners-guide-to-cron-jobs/, https://dev.mysql.com/doc/refman/5.7/en/backup-and-recovery.html]
* Define a cron job that uses Django's `clearsessions` administrative command to regularly clean expired sessions out of the django_session table and prevent it from growing excessively in the project database [https://www.ostechnix.com/a-beginners-guide-to-cron-jobs/, https://docs.djangoproject.com/en/2.0/topics/http/sessions/#clearing-the-session-store]
* Create and use an Admin-targeted view that performs bulk deletion of User/Registry instances based on specific criteria (e.g. last_login) to ensure that the usermerge_user and usermerge_registry tables do not grow excessively in the project database - see models.py
* Create and run unit tests [https://docs.djangoproject.com/en/2.0/topics/testing/]
* Make better use of the Django admin site capabilities
* Make better use of Django's model validation capabilities [https://docs.djangoproject.com/en/2.0/ref/models/instances/]
* Utilize Django's password upgrading and validation schemes [https://docs.djangoproject.com/en/2.0/topics/auth/passwords/]
* Enhance security, e.g. enable HTTPS/SSL protection, enable Host header validation, etc. [https://docs.djangoproject.com/en/2.0/topics/security/]
