"""
Django settings for slub project.

Generated by 'django-admin startproject' using Django 2.0.4.

For more information on this file, see
https://docs.djangoproject.com/en/2.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.0/ref/settings/

For more information on deploying Django (with Apache and mod_wsgi), see
https://docs.djangoproject.com/en/2.0/howto/deployment/ and README.md
"""

import concurrent_log_handler, logging.config, os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '9ni0eujs0@5d$1+a5hm+o&%7y(w^&(&o9gr5$4c&i3ow%sttjf'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'usermerge.apps.UsermergeConfig',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'usermerge.middleware.AuthenticationMiddleware', # instead of default 'django.contrib.auth.middleware.AuthenticationMiddleware'
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'slub.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'slub.wsgi.application'

SECURE_BROWSER_XSS_FILTER = True

SECURE_CONTENT_TYPE_NOSNIFF = True

X_FRAME_OPTIONS = 'DENY'


# Authentication
# https://docs.djangoproject.com/en/2.0/topics/auth/

AUTHENTICATION_BACKENDS = ['django.contrib.auth.backends.ModelBackend', 'usermerge.backends.AppUserModelBackend']


# Logging
# https://docs.djangoproject.com/en/2.0/topics/logging/
# * django.utils.log module (default configuration dictionary, setup process and extensions): https://docs.djangoproject.com/en/2.0/_modules/django/utils/log/
# * Replacement of the default configuration dictionary and setup process with appropriate custom ones:
#   - https://www.caktusgroup.com/blog/2015/01/27/Django-Logging-Configuration-logging_config-default-settings-logger/
#   - https://lincolnloop.com/blog/django-logging-right-way/
# * Concurrent/safe logging to files from multiple processes via the concurrent-log-handler module (supports both Windows and POSIX systems):
#   - https://pypi.org/project/concurrent-log-handler/
#   - https://bugs.python.org/issue4749
#   - The module provides an optional multi-threaded queue logging feature to perform logging in the background. The feature requires Python 3 to run and uses the standard QueueHandler and QueueListener logging classes to create background threads (one for each logger with configured handlers) that handle the logging, thus letting the foreground threads, i.e. those who initiate the logging, handle the requests quickly without blocking for write locks on the log files. It should be used after the logging configuration is set up. The latter is set up during the (usermerge) application setup, the application is set up each time it is loaded and, in production, it is loaded each time a child process is created by the main Apache process (see wsgi.py, https://github.com/django/django/blob/master/django/core/wsgi.py , https://github.com/django/django/blob/master/django/__init__.py and https://modwsgi.readthedocs.io/en/develop/user-guides/processes-and-threading.html). Since the logging configuration is set up for each child process created by the main Apache process, the feature would create too many background threads that could potentially interfere with the operation of Apache and mod_wsgi, which are responsible for managing processes and threads in production, in unexpected ways (https://groups.google.com/forum/#!topic/modwsgi/NMgrti4o9Bw). Therefore, it is recommended that the feature is NOT used, especially in production.
# * Brief summary of the below defined configuration:
#   - If the DEBUG setting is True, i.e. during development, ALL log records/messages are output to the console/terminal (sys.stderr stream) where the Django development server has been run via the runserver command.
#   - If the DEBUG setting is False, i.e. in production (and sometimes also during development to check the in-production functionality of the slub project without having to deploy it directly), ALL log records/messages, EXCEPT FOR the DEBUG ones, are output to log files that exist in the production_logs directory (this directory should exist exactly under the userbase Git repository to keep the existing production log files independent of any modifications to the project/application files). General INFO and WARNING messages are output to general_info_and_warnings.log, ERROR and CRITICAL messages are output to errors.log and INFO messages logged on the usermerge.views.recover_user_profile logger are output to changes_of_credentials_during_profile_recovery.log, while DEBUG messages are NOT output at all. Since the Django development server is inactive, NO messages are logged on the django.server logger for output to the sys.stderr stream. It is worth mentioning that mod_wsgi intercepts the sys.stdout and sys.stderr streams and redirects the output to the Apache error log (https://modwsgi.readthedocs.io/en/develop/user-guides/debugging-techniques.html).
# * Detailed configuration and usage examples: https://www.webforefront.com/django/setupdjangologging.html

PRODUCTION_LOGS_DIR = os.path.abspath(os.path.join(BASE_DIR, '..', 'production_logs/'))

# Ensure that the production_logs directory exists in production (if it does not, create it) to avoid raising
# "No such file or directory" FileNotFoundError exceptions when concurrent-log-handler attempts to output
# messages to the production log files (https://stackoverflow.com/questions/32133856/logger-cannot-find-file).
if not (DEBUG or os.path.exists(PRODUCTION_LOGS_DIR)):
    os.mkdir(PRODUCTION_LOGS_DIR, 0o755) # http://blog.iqandreas.com/ubuntu/what-are-the-default-permissions-for-files-in-ubuntu/

def LOG_LEVEL_IS_LOWER_THAN_ERROR(log_record):
    # This is the callback function of the exclude_errors log filter.
    # It is used to reject the ERROR and CRITICAL log records/messages that pass through the filter.
    if log_record.levelname in ['ERROR', 'CRITICAL']:
        return False
    else:
        return True

LOGGING_CONFIG = None

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse',
        },
        'require_debug_true': {
            '()': 'django.utils.log.RequireDebugTrue',
        },
        'exclude_errors': {
            '()': 'django.utils.log.CallbackFilter',
            'callback': LOG_LEVEL_IS_LOWER_THAN_ERROR,
        },
    },
    'formatters': {
        'django.server': {
            '()': 'django.utils.log.ServerFormatter',
            'format': '[%(server_time)s] %(message)s',
        },
        'verbose': {
            'format': '[%(asctime)s,%(msecs)d <%(levelname)s> - path: %(pathname)s, line: %(lineno)d,'
                      ' func: %(funcName)s, logger: %(name)s, process/thread: %(process)d/%(thread)d] %(message)s',
            'datefmt': '%d/%b/%Y %H:%M:%S',
        },
        'standard': {
            'format': '[%(asctime)s,%(msecs)d <%(levelname)s> - file: %(filename)s, line: %(lineno)d,'
                      ' func: %(funcName)s, logger: %(name)s] %(message)s',
            'datefmt': '%d/%b/%Y %H:%M:%S',
        },
        'laconic': {
            'format': '[%(asctime)s,%(msecs)d <%(levelname)s> - file: %(filename)s, line: %(lineno)d] %(message)s',
            'datefmt': '%d/%b/%Y %H:%M:%S',
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'filters': ['require_debug_true'],
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
        'django.server': {
            'level': 'INFO',
            'class': 'logging.StreamHandler',
            'formatter': 'django.server',
        },
        'file_for_general_info_and_warnings': {
            'level': 'INFO',
            'filters': ['require_debug_false', 'exclude_errors'],
            'class': 'logging.handlers.ConcurrentRotatingFileHandler',
            'filename': os.path.join(PRODUCTION_LOGS_DIR, 'general_info_and_warnings.log'),
            'maxBytes': 1024 * 100, # 100 KB
            'backupCount': 5,
            'formatter': 'verbose',
        },
        'file_for_errors': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'logging.handlers.ConcurrentRotatingFileHandler',
            'filename': os.path.join(PRODUCTION_LOGS_DIR, 'errors.log'),
            'maxBytes': 1024 * 200, # 200 KB
            'backupCount': 5,
            'formatter': 'verbose',
        },
        'file_for_changes_of_credentials_during_profile_recovery': {
            'level': 'INFO',
            'filters': ['require_debug_false'],
            'class': 'logging.handlers.ConcurrentRotatingFileHandler',
            'filename': os.path.join(PRODUCTION_LOGS_DIR, 'changes_of_credentials_during_profile_recovery.log'),
            'maxBytes': 1024 * 20, # 20 KB
            'backupCount': 5,
            'formatter': 'laconic',
        },
    },
    'root': {
        'level': 'DEBUG' if DEBUG else 'INFO',
        'handlers': ['console', 'file_for_general_info_and_warnings', 'file_for_errors'],
    },
    'loggers': {
        'django': {
            'level': 'DEBUG' if DEBUG else 'INFO',
            'handlers': ['console', 'file_for_general_info_and_warnings', 'file_for_errors'],
            'propagate': False,
        },
        'django.server': {
            'level': 'INFO',
            'handlers': ['django.server'],
            'propagate': False,
        },
        'usermerge': {
            'level': 'DEBUG' if DEBUG else 'INFO',
            'handlers': ['console', 'file_for_general_info_and_warnings', 'file_for_errors'],
            'propagate': False,
        },
        'usermerge.views.recover_user_profile': {
            'level': 'INFO',
            'handlers': ['console', 'file_for_changes_of_credentials_during_profile_recovery'],
            'propagate': False,
        },
    },
}

logging.config.dictConfig(LOGGING)


# Database
# https://docs.djangoproject.com/en/2.0/ref/settings/#databases
# * MySQL notes: https://docs.djangoproject.com/en/2.0/ref/databases/#mysql-notes

# If the database tables have NOT been created yet and the database server's default storage engine is MyISAM (MySQL 5.5.4 and earlier),
# SET_DEFAULT_STORAGE_ENGINE_TO_INNODB should be set to True. The default storage engine is thus set to InnoDB upon connecting to the
# database via the init_command option (it is reverted back to MyISAM when the established connection is closed). The corresponding query
# is needed ONLY during table creation. After the tables have been created, it should be omitted from future connections by setting
# SET_DEFAULT_STORAGE_ENGINE_TO_INNODB back to False.
SET_DEFAULT_STORAGE_ENGINE_TO_INNODB = False # see README.md

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.mysql',
        'NAME': 'usermergeDB',
        'USER': 'root',
        'PASSWORD': 'SET_ROOT_PASSWORD_HERE', # see README.md
        'HOST': '',
        'PORT': '',
        'OPTIONS': {
           # The charset option is set to utf8mb4 and the collation_connection variable in the init_command option is set to
           # utf8mb4_0900_ai_ci for MySQL 8.0, utf8mb4_unicode_520_ci for MySQL 5.6/5.7 or utf8mb4_unicode_ci for MySQL 5.5
           # in order to better support the representation and sorting/comparison of Unicode data in the database.
           # For more information, see
           # https://stackoverflow.com/questions/43644218/why-is-table-charset-set-to-utf8mb4-and-collation-to-utf8mb4-unicode-520-ci?rq=1 ,
           # https://stackoverflow.com/questions/38363566/trouble-with-utf-8-characters-what-i-see-is-not-what-i-stored and
           # https://mathiasbynens.be/notes/mysql-utf8mb4 .
            'charset': 'utf8mb4',
 # The sql_mode option includes the same modes as the default SQL mode in MySQL 5.7 (https://dev.mysql.com/doc/refman/5.7/en/sql-mode.html).
 # Even if it is changed at some later point, it should ALWAYS include the STRICT_TRANS_TABLES mode.
            'sql_mode': 'ONLY_FULL_GROUP_BY,STRICT_TRANS_TABLES,NO_ZERO_IN_DATE,NO_ZERO_DATE,'
                        'ERROR_FOR_DIVISION_BY_ZERO,NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION',
            'init_command': 'SET default_storage_engine = InnoDB;' if SET_DEFAULT_STORAGE_ENGINE_TO_INNODB else ''
                            'SET collation_connection = utf8mb4_unicode_520_ci;', # see README.md
        },
    },
}


# Password validation
# https://docs.djangoproject.com/en/2.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.0/topics/i18n/

LANGUAGE_CODE = 'el-gr'

TIME_ZONE = 'Europe/Athens'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.0/howto/static-files/

STATIC_URL = '/static/'


# Local settings file
# As seen in README.md, when the present settings.py file is fetched/downloaded locally from the remote Git repository, some general settings, e.g. SECRET_KEY, DEBUG, ALLOWED_HOSTS and DATABASES['default']['PASSWORD'], need to be changed manually for the project/application to work properly in the local environment, e.g. the server (in production). To facilitate this procedure and avoid the manual changes in the present file on a regular basis, an auxilliary local_settings.py file can be used. This file should then reside in the same directory as the present file and contain the appropriate local values for the general settings that need to be changed. If the local_settings.py file exists, the local settings are imported to the present file, thus overriding the respective general ones. Otherwise (an ImportError exception is raised), no import occurs and the general settings of the present file are normally used - in this case, some manual changes need to be made. It is worth mentioning that if a local_settings.py file is used, it should never be pushed/stored to the remote Git repository for security reasons. Therefore, it should be deleted altogether, be omitted from the Git adding/staging step or be inserted to the .gitignore file of the local Git repository before any project/application changes are pushed to the remote one. For more information, see the "CHECKLIST AND TIPS FOR PROJECT DEPLOYMENT" section of README.md (specifically https://www.agileleaf.com/blog/a-better-way-to-manage-settings-py-in-your-django-projects/ and https://simpleisbetterthancomplex.com/tips/2017/07/03/django-tip-20-working-with-multiple-settings-modules.html).
try:
    from .local_settings import *
except ImportError:
    pass

