from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from functools import wraps

# Create your decorators here.
# There are many different types of decorators in Django, such as view decorators (https://docs.djangoproject.com/en/2.0/topics/http/decorators/), authentication decorators (https://docs.djangoproject.com/en/2.0/topics/auth/default/), utility decorators (https://docs.djangoproject.com/en/2.0/ref/utils/), etc. For more information, search in https://docs.djangoproject.com/en/2.0/search/?q=decorator .

# For more information on decorator definition and usage, see https://realpython.com/primer-on-python-decorators/ .
# For more information on view-related subjects, e.g. request/response objects, exceptions that trigger built-in error views, default and application user models (auth.User and usermerge.{User/Admin}), named URL patterns, etc., see views.py .

def User_login_required(view):
    """
    Ensure that the current user has logged in and that his/her model is usermerge.User in order to execute the wrapped view. If he/she
    has not logged in (his/her model is AnonymousUser), redirect him/her to the login page. If he/she has logged in but his/her model is
    either auth.User or usermerge.Admin, show him/her an HTTP 403 (Forbidden) page by raising a PermissionDenied exception.
    """
    def User_model_required(func):
        @wraps(func)
        def wrapper(request, *args, **kwargs):
            if request.user._meta.label == 'usermerge.User':
                return func(request, *args, **kwargs)
            else: # either auth.User or usermerge.Admin model
                raise PermissionDenied
        return wrapper
    # The login and user model checks are performed by the login_required() and User_model_required() decorators respectively.
    return login_required(User_model_required(view), None, 'default_login')

def Admin_login_required(view):
    """
    Ensure that the current user has logged in and that his/her model is usermerge.Admin in order to execute the wrapped view. If he/she
    has not logged in (his/her model is AnonymousUser), redirect him/her to the login page. If he/she has logged in but his/her model is
    either auth.User or usermerge.User, show him/her an HTTP 403 (Forbidden) page by raising a PermissionDenied exception.
    """
    def Admin_model_required(func):
        @wraps(func)
        def wrapper(request, *args, **kwargs):
            if request.user._meta.label == 'usermerge.Admin':
                return func(request, *args, **kwargs)
            else: # either auth.User or usermerge.User model
                raise PermissionDenied
        return wrapper
    # The login and user model checks are performed by the login_required() and Admin_model_required() decorators respectively.
    return login_required(Admin_model_required(view), None, 'default_login')

