"""
Populate usermergeDB (see models.py) with test data to check the correctness of the usermerge application features.
"""

from django.contrib.auth.hashers import make_password
# Use "usermerge.models" instead of ".models" as the import source module to avoid raising a
# "No module named 'django.core.management.commands.models'" ImportError exception when this
# script is executed via the interactive shell (see README.md).
from usermerge.models import Admin, Platform, Registry, User

# Populate the usermerge_admin table with test Admin instances.
admin_hashed_password = make_password('7wonders')
admin1 = Admin.objects.create(first_name = 'Νικόλαος', last_name = 'Παπασπύρου', email = 'nikolaos@softlab.ntua.gr',
                              username = 'nikolaos', password = admin_hashed_password)
admin2 = Admin.objects.create(first_name = 'Ευστάθιος', last_name = 'Ζάχος', email = 'eustathios@corelab.ntua.gr',
                              username = 'eustathios', password = admin_hashed_password)

# Populate the usermerge_platform table with test Platform instances.
novice = Platform.objects.create(name = 'Novice')
grader = Platform.objects.create(name = 'Grader')
moodle = Platform.objects.create(name = 'Moodle')
plgrader = Platform.objects.create(name = 'PLgrader')

# Populate the usermerge_user table with test User instances.
# The user1 and user2 instances are created with filled out fields. This creation method can SOMETIMES be used during development to
# assist testing/debugging, but it is NOT generally accepted and should ALWAYS be avoided in production. On the other hand, creating
# User instances with empty fields, such as user3, is the accepted method and should ALWAYS be preferred both during development and
# in production.
user1 = User.objects.create(first_name = 'Γεώργιος', last_name = 'Καζελίδης', ece_id = '03199999',
                            email = 'gkazelid@undergraduate.ece.ntua.gr')
user2 = User.objects.create(first_name = 'Ζαχαρίας', last_name = 'Δόγκανος', email = 'zdogkanos@undergraduate.ece.ntua.gr')
user3 = User.objects.create()

# Populate the usermerge_registry table with test Registry instances.
user_hashed_password = make_password('password')
registry1 = Registry.objects.create(user = user1, platform = novice, username = 'pi99b999', password = user_hashed_password)
registry2 = Registry.objects.create(user = user2, platform = moodle, username = 'moodlemoot', password = user_hashed_password)
registry3 = Registry.objects.create(user = user3, platform = plgrader, username = 'proglang', password = user_hashed_password)

